function COR2AVW(fname,volfname,hdrflag)
% Converts a list of COR-* files to an AVW volume
% COR2AVW function to translate MGH-style COR
% files produces with the Dale flattening 
% software to an AVW volume. COR files are 
% always have 8-bit unsigned char datatype,
% so the output volume will have that here
% as well.
% Example: COR2AVW('images.txt','TestVol','y')
% will read in COR files listed in images.txt 
% and output files TestVol.hdr and TestVol.img
% Written 1/21/99: Timothy M. Ellmore, LBC/NIMH

% Converts a list of COR-* files to an AVW volume

% for MGH flattening COR files have these dimensions
xdim = 256;
ydim = 256;
zdim = 256;

% datatype is always 8-bit uchar for MGH COR files
type = 'uchar';

[CORfiles, NumCOR] = ReadPaths(fname);

% volume store
Volume = zeros(xdim,ydim,zdim);

% read in COR files and place each in volume matrix

for i = 1:zdim

  % get current COR file name
  CORname = char(CORfiles(i));
  
  % open the file
  [fid, errormessage] = fopen(CORname, 'rb');
  % check for problems opening the file
  if(fid == -1)
    disp('*********************')
    disp(['Problem opening file: ' CORname])
    disp('fopen returned the following:')
    disp(errormessage)
    disp('Exiting matlab . . . ')
    disp('*********************')
    exit
  end

  % read the data
  disp(['Reading: ' CORname])
  COR = fread(fid, [xdim * ydim], type);

  %close the file
  fclose(fid);

  % display max and min for current COR file
  disp(['max: ' num2str(max(COR(:))) ', min: '  num2str(min(COR(:))) ])

  % form 2D sliceplane matrix 
  COR = reshape(COR,xdim,ydim);
  
  % place matrix into volume
  % matlab has column, row precedence so we must exchange dims
  Volume(:,:,i) = fliplr(COR(:,:));
  
end

% write volume matrix as 8-bit unsigned char

disp(['writing file ' num2str(volfname) ', max: ' num2str(max(Volume(:))) ', min: ' num2str(min(Volume(:))) ])
if(hdrflag == 'y')
 imgfname = sprintf('%s%s',volfname,'.img');
 fid = fopen(imgfname, 'w');
else
 fid = fopen(volfname, 'w');
end

fwrite(fid, Volume(:), type);
fclose(fid);

% check to see whether we can make header
if(hdrflag == 'y')

disp('Making header . . . ')

disp(['/home/joem/bin/makeheader4 ' volfname '.hdr ' num2str(xdim) ' ' num2str(ydim) ' ' num2str(zdim) ' 1 1 1 1 8 255 0 255 0 2 1']);

eval(['!/home/joem/bin/makeheader4 ' volfname '.hdr ' num2str(xdim) ' ' num2str(ydim) ' ' num2str(zdim) ' 1 1 1 1 8 255 0 255 0 2 1']);

end




