function [FAs, MDs, EIG3s,estTime] =TensorEstimationFunction(data,gradientTable,bvalue,roiMask,textProgress,options)
% function [FAs, MDs, EIG3s] =TensorEstimationFunction(data,gradientTable,roiMask,textProgress,options)
%
% This is a generic tensor estimation procedure. It must be implemented by
% the user to evaluate any particular method. 
%
% Inputs:
%   data : Image Data (256x256x(1+Ngrad)). I minimally weighted volume
%       followed by Ngrad diffusion weighted volumes.
%   gradientTable : Diffusion Wieghting Table (Ngradx3). X (dim 1)-Y (dim
%       2)-Z (through plane) diffusion weighting directions
%   bvalue : b-value of the experiment (s/mm2)
%   roiMask : Binary Mask (256x256) of voxels that will be used to compute
%       performance statistics. May be ignored.
%   textProgress : A progress indicator that the function may display or
%       write to a log file.
%   options : any additional options specified in a structure to the
%       control loop.
%
% Outputs: 
%   FAs : Computed Fractional Anisotropy (FA) (256x256)
%   MDs : Computed Mean Diffusivity (MD) (256x256)
%   EIG3s : Computed Eigenvalues (256x256x3). EIG3S(:,:,1) corresponds to
%       the largest eigenvalue.
%   estTime : The number of seconds used during the tensor estimation step.
%
% Bennett Landman, April 30, 2007
error('Not Implemented');