%%%%% BEGIN: USER SPECIFIED PARAMETER
addpath(genpath(fileparts(which('example.m')))); % add this directory and subdirectories to the path.
OutputPath = [pwd filesep 'simdata'];
%%%%% END: USER SPECIFIED PARAMETER

%%%%% BEGIN: SIMULATION PARAMETERS
NumberOfSimulatedDatasets = 2;
SimulatedNoiseLevels = logspace(1,6,75);
GradientTable = load('jones30.txt');
% Ensure normalized gradient table
GradientTable = GradientTable ./repmat(sqrt(sum(GradientTable .^2,2)),[1 3]);

fp = fopen('data_float32le_256x256x31.raw','rb','ieee-le');
HighSNRData = reshape(fread(fp,'float'),[256 256 31]);
fclose(fp);
DataMask = HighSNRData(:,:,1)>15000;
bvalue = 1000; %s/mm2
%%%%% END: SIMULATION PARAMETERS


% Perform Tensor Simulations Over Slice
%[biasFA,varFA,meanFA,biasMD,varMD,meanMD,meanEig,varEig,SNR] = ...
%    simulateTensorSlice(@TensorEstimationFunction,HighSNRData, GradientTable,bvalue,DataMask, SimulatedNoiseLevels, NumberOfSimulatedDatasets);

[biasFA,varFA,meanFA,biasMD,varMD,meanMD,meanEig,varEig,SNR,estTime] = ...
    simulateTensorSlice(@TensorEstimatorAFNI,HighSNRData, GradientTable,bvalue,DataMask, SimulatedNoiseLevels, NumberOfSimulatedDatasets);

% Aggregate metrics over slice
NumVoxels = sum(DataMask(:));
for j=1:length(biasFA)
    f= find(DataMask(:));
    meanSNR(j) = sum(SNR{j}(f))/sum(DataMask(:));

    meanbiasFA(j) = sum(sum(DataMask.*biasFA{j}(:,:)))/NumVoxels;
    meanvarFA(j) = sum(sum(DataMask.*varFA{j}(:,:)))/NumVoxels;
    meanmeanFA(j) = sum(sum(DataMask.*meanFA{j}(:,:)))/NumVoxels;

    meanbiasMD(j) = sum(sum(DataMask.*biasMD{j}(:,:)))/NumVoxels;
    meanvarMD(j) = sum(sum(DataMask.*varMD{j}(:,:)))/NumVoxels;
    meanmeanMD(j) = sum(sum(DataMask.*meanMD{j}(:,:)))/NumVoxels;

end
meanrmsFA = sqrt(meanbiasFA.^2+meanvarFA);
meanrmsMD = sqrt(meanbiasMD.^2+meanvarMD);

clinicalSNR = find((meanSNR>25).*(meanSNR<40));
expandedClinicalSNR = find((meanSNR>20).*(meanSNR<45));

% Compute error metrics; 
metricFArmse = trapz(meanSNR(clinicalSNR),meanrmsFA(clinicalSNR))/(meanSNR(clinicalSNR(end))-meanSNR(clinicalSNR(1)));
metricMDrmse = trapz(meanSNR(clinicalSNR),meanrmsMD(clinicalSNR))/(meanSNR(clinicalSNR(end))-meanSNR(clinicalSNR(1)));

metricFAbias = trapz(meanSNR(clinicalSNR),meanbiasFA(clinicalSNR))/(meanSNR(clinicalSNR(end))-meanSNR(clinicalSNR(1)));
metricMDbias = trapz(meanSNR(clinicalSNR),meanbiasMD(clinicalSNR))/(meanSNR(clinicalSNR(end))-meanSNR(clinicalSNR(1)));

metricFAvar = trapz(meanSNR(clinicalSNR),meanvarFA(clinicalSNR))/(meanSNR(clinicalSNR(end))-meanSNR(clinicalSNR(1)));
metricMDvar = trapz(meanSNR(clinicalSNR),meanvarMD(clinicalSNR))/(meanSNR(clinicalSNR(end))-meanSNR(clinicalSNR(1)));

% Fit sigmoids to the RMS errors

x = fminsearch(@(x) sum((meanrmsFA(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), [1 1 1 1]*10);
x = fminsearch(@(x) sum((meanrmsFA(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), x);
x = fminsearch(@(x) sum((meanrmsFA(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), x);
sigmoidrmsFA = fminsearch(@(x) sum((meanrmsFA(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), x);


x = fminsearch(@(x) sum((meanrmsMD(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), [1 1 1 1]*10);
x = fminsearch(@(x) sum((meanrmsMD(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), x);
x = fminsearch(@(x) sum((meanrmsMD(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), x);
sigmoidrmsMD = fminsearch(@(x) sum((meanrmsMD(expandedClinicalSNR)-sigmoid(meanSNR(expandedClinicalSNR),x)).^2), x);

figure(1); 
subplot(2,1,1);
h=plot(meanSNR,meanrmsFA,'.',meanSNR(expandedClinicalSNR),sigmoid(meanSNR(expandedClinicalSNR),sigmoidrmsFA),'-k');
set(h,'linewidth',2)
ylabel('RMSE FA')
xlabel('SNR (dB)')
legend('Mean RMSE','Sigmoid Fit')
title('Root Mean Squared Error (RMSE)')
subplot(2,1,2);
h=plot(meanSNR,meanrmsMD,'.',meanSNR(expandedClinicalSNR),sigmoid(meanSNR(expandedClinicalSNR),sigmoidrmsMD),'-k');
set(h,'linewidth',2)
ylabel('RMSE MD')
xlabel('SNR (dB)')
legend('Mean RMSE','Sigmoid Fit')

% Report summary error metrics
disp('--------------------------------------------------------------------------')
disp('Summary Measures');
disp('--------------------------------------------------------------------------')
disp('Mean Estimation Time')
disp(['   Slice: ' sprintf('%.3e',estTime)]);
disp('RMS Error');
disp(['   FA:    ' sprintf('%3.2f',metricFArmse*1000) ' [FA]*1000']);        
disp(['   MD:    ' sprintf('%3.2f',metricMDrmse*1000*1000) ' um2/s*1000']); 
disp('Bias');
disp(['   FA:    ' sprintf('%3.2f',metricFAbias*1000) ' [FA]*1000']);
disp(['   MD:    ' sprintf('%3.2f',metricMDbias*1000*1000) ' um2/s*1000']);
disp('Std. Dev.');
disp(['   FA:    ' sprintf('%3.2f',sqrt(metricFAvar)*1000) ' [FA]*1000]']);
disp(['   MD:    ' sprintf('%3.2f',sqrt(metricMDvar)*1000*1000) ' um2/s*1000']);
disp('Sigmoid Form: RMSE = P(1)*(1+P(2)*exp(-SNR/P(4)))./(1+P(3)*exp(-SNR/P(4)))');
disp('Note: SNR are in power dB');
disp(['Sigmoid Fit FA: ' sprintf('%.2e ',sigmoidrmsFA)]);
disp(['Sigmoid Fit MD: ' sprintf('%.2e ',sigmoidrmsMD)]);


save example